#include <Arduino.h>
#include <U8x8lib.h>
#include "MFRC522_I2C.h"
#include <wire.h>

#define ButnPin D1
#define SolenoidPin D10
#define SolenoidOnTime 500
const int8_t MaxUidLen = 10;

U8X8_SSD1306_128X32_UNIVISION_SW_I2C oled(/* clock=SCL*/ D2, /* data=SDA*/ D3, /* reset=*/ U8X8_PIN_NONE); //For external little Oled Display
MFRC522 mfrc522(0x28); // Create MFRC522 instance.

struct Entity
{
	char name[32];
	MFRC522::Uid uid;
};

Entity entities[] = {{"Frank Furter", {4, {0xb0, 0xc8, 0x10, 0x13}, 8}},
					{"Ali Baba", {4, {0xF5, 0x76, 0x7F, 0x3A}, 8}}};

int8_t validate(int size, MFRC522::Uid uid)
{
	int arraySize = sizeof(entities) / sizeof(Entity);
	for (int i=0; i < arraySize; i++)
	{
		if (entities[i].uid.size == size)
		{
			int matchSize = 0;
			for (int x = 0; x < size; x++)
			{
				if (entities[i].uid.uidByte[x] == uid.uidByte[x])
				{
					matchSize++;
				}
			}
			if (matchSize == entities[i].uid.size)
			{ 
				return i; //Return record number found
			}
		}
	}
	return -1;
}

void oledReset()
{
	oled.initInterface();
	 oled.begin();
	 oled.setFont(u8x8_font_chroma48medium8_r);
	 oled.setCursor(0, 0);
}

void setup()
{
	Serial.begin(9600);
	int8_t arraySize = sizeof(entities) / sizeof(Entity);

	pinMode(ButnPin, INPUT_PULLUP);
	pinMode(SolenoidPin, OUTPUT);
	digitalWrite(SolenoidPin, LOW);
	Wire.begin(); // Wire init, adding the I2C bus.
	oledReset();
	 oled.println("Waiting for");
	 oled.println("RFID Module");
	mfrc522.PCD_Init(); // Init MFRC522.
 	oled.clear();
	oled.println("RFID Ready...");
}

/// @brief Energizes the Solenoid (i.e. Activate it).
/// @param ms Number of milliseconds to energize.
void energizeSolenoid(int ms)
{
	digitalWrite(SolenoidPin, HIGH);
	delay(ms);
	digitalWrite(SolenoidPin, LOW);
}

void loop()
{
	if (digitalRead(ButnPin) == LOW)
	{
		oledReset();
		oled.println("Manual");
		oled.println("Activation");
		energizeSolenoid(SolenoidOnTime);
	}
	else if (mfrc522.PICC_IsNewCardPresent())
	{
		if (!mfrc522.PICC_ReadCardSerial())
		{
			delay(200);
			return;
		}
		else
		{
			oledReset();
			int8_t found = validate(mfrc522.uid.size, mfrc522.uid);
			if (found >= 0)
			{
				oled.println("Welcome");
				oled.println(entities[found].name);
				energizeSolenoid(SolenoidOnTime);
			}
			else
			{
				oled.println("Invalid");
				oled.println("Card/ID");
			}
			delay(333);
		}
	}
}
